// ======================================================================
//
// Palette.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef PALETTE_H
#define PALETTE_H

// ======================================================================

class Iff;
class PaletteData;

// ======================================================================

class Palette
{
	friend class PaletteList;

private:

	char                        *name;
	Palette                     *next;

	mutable int                  users;
	int                          serial;

	int                          entries;
	RGB                         *rgb;
	RGB                         *raw;
	
	mutable PaletteData         *data;
	
private:

	Palette(void);
	Palette(const Palette &);
	Palette &operator=(const Palette &);

	void gammaCorrect(void);

private:

	explicit Palette(const char *name);
	explicit Palette(Iff &iff);
	Palette(int num, const RGB *newRGB);
	~Palette(void);

	void init(void);
	void load(int num, const RGB *newRGB);
	void load(Iff &iff);

	void loadData(void) const;
	void unloadData(void) const;

public:

	void addUser(void) const;

	// inspection
	int        getEntries(void) const;
	const RGB *getRGB(int offset=0) const;
	const RGB *getRawRGB(int offset=0) const;

	u_int8     findClosestColor(int r, int g, int b) const;

	// implementation-specific-data routines
	virtual PaletteData *getData(void) const;
};

class PaletteList
{
private:

	// prevent objects of this class from being created
	PaletteList(void);
	PaletteList(const PaletteList &);
	PaletteList &operator =(const PaletteList &);

private:

	static real gammaCorrection;

	static Palette *firstNamed;
	static Palette *firstAnonymous;

	static void remove(Palette *p, Palette **list);

public:

	static const Palette *fetch(const char *palname);
	static const Palette *fetch(Iff &iff);
	static const Palette *create(int numEntries, const RGB *rgb);
	
	static void           release(const Palette *p);
	static void           unloadAllData(void);

	static void setGammaCorrection(real newGamma);
	static void adjustGammaCorrection(real deltaGamma);
	static real getGammaCorrection(void);
};

// ======================================================================

inline const RGB *Palette::getRGB(int offset) const
{
	return rgb+offset;
}

// ----------------------------------------------------------------------

inline const RGB *Palette::getRawRGB(int offset) const
{
	return raw+offset;
}

// ----------------------------------------------------------------------

inline int Palette::getEntries(void) const
{
	return entries;
}

// ----------------------------------------------------------------------

inline void Palette::addUser(void) const
{
	users++;
}

// ======================================================================

#endif
